/*****************************************************************
 * File:		$RCSfile: ifdhandler.c,v $
 * Date:		$Date: 2005/05/23 09:30:08 $
 * Author:		$Author: haikun $
 * Revision:	$Revision: 1.12 $
 *
 * Purpose:		This provides reader specific low-level calls
 * 				for the ArgosMini II (USB) reader of Todos Data System AB.
 * License:		See file LICENSE
 *******************************************************************/

#include <pcsclite.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>

#include "AgmII.h"

/*****************************************************************************
*
*****************************************************************************/

static reader readerData[MAX_READER_NUM];


/*****************************************************************************
*
*****************************************************************************/
static void closeDriver(DWORD Lun)
{
	int readerNum = (Lun & 0xFFFF0000) >> 16;

	CloseUSB(&readerData[readerNum]);

	return;
}

/*****************************************************************************
*
*****************************************************************************/
RESPONSECODE IFDHCreateChannel(DWORD Lun, DWORD Channel)
{

	/* Lun - AgmLogical Unit Number, use this for multiple card slots
	 * or multiple readers. 0xXXXXYYYY -  XXXX multiple readers,
	 * YYYY multiple slots. The resource manager will set these
	 * automatically.  By default the resource manager loads a new
	 * instance of the driver so if your reader does not have more than
	 * one smartcard slot then ignore the Lun in all the functions.
	 * Future versions of PC/SC might support loading multiple readers
	 * through one instance of the driver in which XXXX would be important
	 * to implement if you want this.
	 */

	/* Channel - Channel ID.  This is denoted by the following:
	 * 0x000001 - /dev/pcsc/1
	 * 0x000002 - /dev/pcsc/2
	 * 0x000003 - /dev/pcsc/3
	 *
	 * USB readers may choose to ignore this parameter and query
	 * the bus for the particular reader.
	 */

	/* This function is required to open a communications channel to the
	 * port listed by Channel.  For example, the first serial reader on COM1 would
	 * link to /dev/pcsc/1 which would be a sym link to /dev/ttyS0 on some machines
	 * This is used to help with intermachine independance.
	 *
	 * Once the channel is opened the reader must be in a state in which it is possible
	 * to query IFDHICCPresence() for card status.
	 *
	 * returns:
	 *
	 * IFD_SUCCESS
	 * IFD_COMMUNICATION_ERROR
	 */

	int readerNum = (Lun & 0xFFFF0000) >> 16;

#ifdef AGMII_DEBUG
	char name[30];
#endif

	/* open the communication */
	if (OpenUSB(readerData, &readerData[readerNum]) == TRUE)
	{
#ifdef AGMII_DEBUG
		AgmLog1(AGM_LOG_DEBUG, " opened for IO.\n");
#endif
	}
	else
	{
#ifdef AGMII_DEBUG
		AgmLog2(AGM_LOG_DEBUG,
				  "IFDHCreateChannel: Error! Could not open %s. Existing.\n",
				  name);
#endif
		return IFD_COMMUNICATION_ERROR;
	}

    /* reader open, initial useful parameters */
	int retVal;
	retVal = pthread_mutex_init(&(readerData[readerNum].semaphore), NULL);
	if (retVal)
	{
		AgmLog1(AGM_LOG_DEBUG,
				  "Argos Mini II Reader : pthread_mutex_init fail\n");
	}

    readerData[readerNum].ifsAdjust = FALSE;

#ifdef AGMII_DEBUG
	AgmLog1(AGM_LOG_DEBUG,
			  "Argos Mini II Reader : exisiting IFDHCreateChannel\n");
#endif

	return IFD_SUCCESS;
}


/*****************************************************************************
*
*****************************************************************************/
RESPONSECODE IFDHCloseChannel(DWORD Lun)
{

	/* This function should close the reader communication channel
	 * for the particular reader.  Prior to closing the communication channel
	 * the reader should make sure the card is powered down and the terminal
	 * is also powered down.
	 *
	 * returns:
	 *
	 * IFD_SUCCESS
	 * IFD_COMMUNICATION_ERROR
	 */

#ifdef AGMII_DEBUG
	AgmLog3(AGM_LOG_DEBUG,
			  "Argos Mini II Reader : entering IFDHCloseChannel Lun = 0x%04x%04x\n",
			  (int) ((Lun >> 16) & 0xFFFF), (int) (Lun & 0xFFFF));
#endif

	closeDriver(Lun);

#ifdef AGMII_DEBUG
	AgmLog1(AGM_LOG_DEBUG,
			  "Argos Mini II Reader : exisiting IFDHCloseChannel\n");
#endif

	return IFD_SUCCESS;
}


/*****************************************************************************
*
*****************************************************************************/
RESPONSECODE IFDHGetCapabilities(DWORD Lun, DWORD Tag,
											PDWORD Length, PUCHAR Value)
{

	/* This function should get the slot/card capabilities for a particular
	 * slot/card specified by Lun.  Again, if you have only 1 card slot and don't mind
	 * loading a new driver for each reader then ignore Lun.
	 *
	 * Tag - the tag for the information requested
	 * example: TAG_IFD_ATR - return the Atr and it's size (required).
	 * these tags are defined in ifdhandler.h
	 *
	 * Length - the length of the returned data
	 * Value  - the value of the data
	 *
	 * returns:
	 *
	 * IFD_SUCCESS
	 * IFD_ERROR_TAG
	 */

#ifdef AGMII_DEBUG
	AgmLog3(AGM_LOG_DEBUG,
			  "Argos Mini II Reader : entering IFDHGetCapabilities Lun = 0x%04x%04x\n",
			  (int) ((Lun >> 16) & 0xFFFF), (int) (Lun & 0xFFFF));
#endif
	int readerNum = (Lun & 0xFFFF0000) >> 16;
// int retVal;
	switch (Tag)
	{
	case TAG_IFD_ATR:
	{
		*Length = readerData[readerNum].card.atr.length;
#ifdef AGMII_DEBUG
		AgmLog2(AGM_LOG_DEBUG, "TAG_IFD_ATR - *Length = %d\n", (int) (*Length));
#endif
		if (*Length)
		{
			memcpy(Value, readerData[readerNum].card.atr.data, *Length);
		}
		break;
	}
	case TAG_IFD_SIMULTANEOUS_ACCESS:
	{
#ifdef AGMII_DEBUG
		AgmLog1(AGM_LOG_DEBUG, "TAG_IFD_SIMULTANEOUS_ACCESS\n");
#endif
		*Length = 1;
		*Value = MAX_READER_NUM;
		break;
	}
	case TAG_IFD_SLOTS_NUMBER:
	{
#ifdef AGMII_DEBUG
		AgmLog1(AGM_LOG_DEBUG, "TAG_IFD_SLOTS_NUMBER\n");
#endif

		*Length = 1;
		*Value = 1;
		break;
	}
	case TAG_IFD_THREAD_SAFE:
	{
#ifdef AGMII_DEBUG
		AgmLog1(AGM_LOG_DEBUG, "TAG_IFD_THREAD_SAFE\n");
#endif
		if (*Length >= 1)
		{
			*Length = 1;
			*Value = 1;
		}
		break;
	}

	default:
#ifdef AGMII_DEBUG
		AgmLog1(AGM_LOG_DEBUG, "default TAG Tag \n");
#endif
		return IFD_ERROR_TAG;
	}

#ifdef AGMII_DEBUG
	AgmLog1(AGM_LOG_DEBUG,
			  "Argos Mini II Reader : exisiting IFDHGetCapabilities\n");
#endif

	return IFD_SUCCESS;
}


/*****************************************************************************
*
*****************************************************************************/
RESPONSECODE IFDHSetCapabilities(DWORD Lun, DWORD Tag,
											DWORD Length, PUCHAR Value)
{

	/* This function should set the slot/card capabilities for a particular
	 * slot/card specified by Lun.  Again, if you have only 1 card slot and don't mind
	 * loading a new driver for each reader then ignore Lun.
	 *
	 * Tag - the tag for the information needing set
	 *
	 * Length - the length of the returned data
	 * Value  - the value of the data
	 *
	 * returns:
	 *
	 * IFD_SUCCESS
	 * IFD_ERROR_TAG
	 * IFD_ERROR_SET_FAILURE
	 * IFD_ERROR_VALUE_READ_ONLY
	 */

#ifdef AGMII_DEBUG
	AgmLog3(AGM_LOG_DEBUG,
			  "Argos Mini II Reader : entering IFDHSetCapabilities Lun = 0x%04x%04x\n",
			  (int) ((Lun >> 16) & 0xFFFF), (int) (Lun & 0xFFFF));
#endif

#ifdef AGMII_DEBUG
	AgmLog1(AGM_LOG_DEBUG,
			  "Argos Mini II Reader : exisiting IFDHSetCapabilities\n");
#endif

	return IFD_SUCCESS;
}


/*****************************************************************************
*
*****************************************************************************/
RESPONSECODE IFDHSetProtocolParameters(DWORD Lun, DWORD Protocol,
													UCHAR Flags, UCHAR PTS1,
													UCHAR PTS2, UCHAR PTS3)
{

	/* This function should set the PTS of a particular card/slot using
	 * the three PTS parameters sent
	 *
	 * Protocol  - 0 .... 14  T=0 .... T=14
	 * Flags     - AgmLogical OR of possible values:
	 * IFD_NEGOTIATE_PTS1 IFD_NEGOTIATE_PTS2 IFD_NEGOTIATE_PTS3
	 * to determine which PTS values to negotiate.
	 * PTS1,PTS2,PTS3 - PTS Values.
	 *
	 * returns:
	 *
	 * IFD_SUCCESS
	 * IFD_ERROR_PTS_FAILURE
	 * IFD_COMMUNICATION_ERROR
	 * IFD_PROTOCOL_NOT_SUPPORTED
	 */

    AgmLog3(AGM_LOG_DEBUG,
			  "Argos Mini II Reader : entering IFDHSetProtocolParameters Lun = 0x%04x%04x\n",
			  (int) ((Lun >> 16) & 0xFFFF), (int) (Lun & 0xFFFF));

    unsigned char pps[ARGMINIUSB_FIFO_LEN];
    unsigned char ppsReply[ARGMINIUSB_FIFO_LEN];
    int retVal, readerNum = (Lun & 0xFFFF0000) >> 16;
    unsigned char SetPTSStr[ARGMINIUSB_FIFO_LEN] = {CommandID,AGMU_PTS_Data_length,0x00};
    bool ppsResult;

    /* set to zero buffer */
    memset( pps, 0, sizeof(pps));
    memset( ppsReply, 0, sizeof(ppsReply));

	// works only for T=0 and T=1
	if (Protocol != SCARD_PROTOCOL_T0 && Protocol != SCARD_PROTOCOL_T1)
	{
		return IFD_PROTOCOL_NOT_SUPPORTED;
	}

    pps[0] = DataID;
    pps[1] = 4;
    pps[2] = 0xFF;
    if ( SCARD_PROTOCOL_T0 == Protocol ) {
        pps[3] |= ATR_PROTOCOL_TYPE_T0;
    }
    else if( SCARD_PROTOCOL_T1 == Protocol ) {
        pps[3] |= ATR_PROTOCOL_TYPE_T1;
    }

	AgmLog2(AGM_LOG_DEBUG, "PTS Flags value %x", Flags);

    /* TA2 present -> specific mode */
    if( SPECIFIC_MODE == GetCurrentMode(&readerData[readerNum].card.atr) )
    {
        if( pps[3] != GetSpecificModeProtocol(&readerData[readerNum].card.atr) )
                return IFD_PROTOCOL_NOT_SUPPORTED;
    }

    if( Flags & IFD_NEGOTIATE_PTS1 )
    {
        /* just use the value pass by argument */
        pps[3] |= 0x10;
        pps[4] = PTS1;
    }
    else {
        unsigned char currentDi;
        pps[3] |= 0x10;
        currentDi = GetDi(&readerData[readerNum].card.atr);
        /*-----------------2005-5-12 14:53------------------
         * Check if reader support the Di value
         * --------------------------------------------------*/
        if( currentDi > 0x03 )
            pps[4] |= GetFi(&readerData[readerNum].card.atr) << 4 | 0x01;  /* using default bautrate */
        else
            pps[4] |= GetFi(&readerData[readerNum].card.atr) << 4 | currentDi;
    }

    pps[5] = pps[2] ^ pps[3] ^ pps[4];

    /* PPS2 and PPS3 are reserved for future use */
    if( Flags & IFD_NEGOTIATE_PTS2 )
    {
        /* just use the value pass by argument */
        pps[3] |= 0x20;
        pps[5] = PTS2;
        pps[6] = pps[2] ^ pps[3] ^ pps[4] ^ pps[5];
        pps[1] += 1;
    }

    if( Flags & IFD_NEGOTIATE_PTS3 )
    {
        /* just use the value pass by argument */
        pps[3] |= 0x40;
        pps[6] = PTS3;
        pps[7] = pps[2] ^ pps[3] ^ pps[4] ^ pps[5] ^ pps[6];
        pps[1] += 1;
    }

    AgmLogXxd(AGM_LOG_DEBUG, "Build PPS block :", pps, ARGMINIUSB_FIFO_LEN);

    /* TA2 absence: negotiate mode */
    if( NEGOTIABLE_MODE == GetCurrentMode(&readerData[readerNum].card.atr) )
    {
        /*-----------------------------------------------------
         * if the requested  protocol is not the first offered protocol
         * or PPT1 is present
         * --------------------------------------------------*/
        if( (pps[3] & 0x0F) != GetFirstOfferedProtocol(&readerData[readerNum].card.atr) ||
                ( pps[3] & 0x10 ) == 0x10)
        {
            int ReadTimeout = 11000;

            SetPTSStr[2] = pps[1];

            // send PTS block first
            retVal = WriteUSB(&readerData[readerNum], pps, AGM_DEFAULT_TIMEOUT);
            if (retVal <= 0)
            {
                AgmLog1(AGM_LOG_DEBUG, "Send PPS block fail");
                return IFD_ERROR_PTS_FAILURE;
            }

            // send SetPTSStr command
            retVal = WriteUSB(&readerData[readerNum], SetPTSStr, AGM_DEFAULT_TIMEOUT);
            if (retVal <= 0)
            {
                AgmLog1(AGM_LOG_DEBUG, "Send SetPTSStr command fail");
                return IFD_ERROR_PTS_FAILURE;
            }

            // Read PPS reply
            retVal = ReadUSB(&readerData[readerNum], ppsReply, ReadTimeout);
            if (retVal > 0)
            {
                AgmLogXxd(AGM_LOG_DEBUG, "PPS reply block :", ppsReply, ARGMINIUSB_FIFO_LEN);
                // Check if package is a AGMU_CardStatus_IND.
                if ( CommandID == ppsReply[0] )
                {
                    if ( AGMU_CardStatus_IND == ppsReply[1] )
                    {
                        if ( 0 == ppsReply[2] )
                        {
                            // No card in reader
                            return IFD_COMMUNICATION_ERROR;
                        }
                    }
                    else
                    {
                        // Data error from reader. Abort!
                        return IFD_COMMUNICATION_ERROR;
                    }
                }
                else
                {
                    if (!DataPackageCheck(ppsReply))
                    {
                        return IFD_COMMUNICATION_ERROR;
                    }

                    // ISO7816 7.4 successfull exchange case 1
                    // check if the pps reply same as pps request
                    if( (ppsReply[1] == pps[1]) &&
                            (memcmp( pps, ppsReply, ppsReply[1]+2) == 0))
                            ppsResult = TRUE;
                    else
                            ppsResult = FALSE;

                    // ISO7816 7.4 successfull exchange case 2
                    if( !ppsResult )
                        if( ( (ppsReply[3] & 0x6F) == (pps[3] & 0x6F) ) &&
                               ( (ppsReply[3] & 0x10) == 0x00 ) )
                               ppsResult = TRUE;
                }
            }
            else
            {
                return IFD_COMMUNICATION_ERROR;
            }
        }
    }

    // set reader parameters
    /* T1 */
    if( SCARD_PROTOCOL_T1 == Protocol )
    {
        readerData[readerNum].card.activeProtocol = SCARD_PROTOCOL_T1;
//        readerData[readerNum].ReadTimeout = ;
    }
    else
    {
        readerData[readerNum].card.activeProtocol = SCARD_PROTOCOL_T0;
//        readerData[readerNum].ReadTimeout = ;
    }

    if( ppsResult )
    {
        // change bautrate
        readerData[readerNum].CurrentDi = pps[4] & 0x0F;
        AgmLog2(AGM_LOG_DEBUG, "readerData[readerNum].CurrentDi %x", readerData[readerNum].CurrentDi);
        return IFD_SUCCESS;
    }
#ifdef AGMII_DEBUG
	AgmLog1(AGM_LOG_DEBUG,
			  "Argos Mini II Reader : exisiting IFDHSetProtocolParameters\n");
#endif

	return IFD_SUCCESS;
}


/*****************************************************************************
*
*****************************************************************************/
RESPONSECODE IFDHPowerICC(DWORD Lun, DWORD Action,
								  PUCHAR Atr, PDWORD AtrLength)
{

	/* This function controls the power and reset signals of the smartcard reader
	 * at the particular reader/slot specified by Lun.
	 *
	 * Action - Action to be taken on the card.
	 *
	 * IFD_POWER_UP - Power and reset the card if not done so
	 * (store the ATR and return it and it's length).
	 *
	 * IFD_POWER_DOWN - Power down the card if not done already
	 * (Atr/AtrLength should
	 * be zero'd)
	 *
	 * IFD_RESET - Perform a quick reset on the card.  If the card is not powered
	 * power up the card.  (Store and return the Atr/Length)
	 *
	 * Atr - Answer to Reset of the card.  The driver is responsible for caching
	 * this value in case IFDHGetCapabilities is called requesting the ATR and it's
	 * length.  This should not exceed MAX_ATR_SIZE.
	 *
	 * AtrLength - Length of the Atr.  This should not exceed MAX_ATR_SIZE.
	 *
	 * Notes:
	 *
	 * Memory cards without an ATR should return IFD_SUCCESS on reset
	 * but the Atr should be zero'd and the length should be zero
	 *
	 * Reset errors should return zero for the AtrLength and return
	 * IFD_ERROR_POWER_ACTION.
	 *
	 * returns:
	 *
	 * IFD_SUCCESS
	 * IFD_ERROR_POWER_ACTION
	 * IFD_COMMUNICATION_ERROR
	 * IFD_NOT_SUPPORTED
	 */


#ifdef AGMII_DEBUG
	AgmLog3(AGM_LOG_DEBUG,
			  "Argos Mini II Reader : entering IFDHPowerICC Lun = 0x%04x%04x\n",
			  (int) ((Lun >> 16) & 0xFFFF), (int) (Lun & 0xFFFF));
#endif

	int retVal, readerNum = (Lun & 0xFFFF0000) >> 16;
	RESPONSECODE pcscResponce = IFD_SUCCESS;
	memset(Atr, 0, MAX_ATR_SIZE);

	lock_mutex(&readerData[readerNum]);

	switch (Action)
	{
	case IFD_POWER_UP:
	case IFD_RESET:
		if( IFD_POWER_UP == Action )
		{
			AgmLog1(AGM_LOG_DEBUG, "Argos Mini II Reader:IFD_POWER_UP \n");
		}
		else
		{
			AgmLog1(AGM_LOG_DEBUG, "Argos Mini II Reader:IFD_RESET \n");
		}
        /* initial t1Params */
        t1_init(&readerData[readerNum].card.t1Params);

		/* Set default baudrate. */
		readerData[readerNum].CurrentDi = 1;

		/* Power on the card. */
		retVal = CardPowerOn(&readerData[readerNum]);

		if( AGM_OK == retVal )
		{
			*AtrLength = readerData[readerNum].card.atr.length;
			memcpy(Atr, readerData[readerNum].card.atr.data, *AtrLength);

            readerData[readerNum].ifsAdjust = FALSE;

            readerData[readerNum].card.t1Params.ifsc = GetT1IFSC(&readerData[readerNum].card.atr);

            unsigned char SetCardDataStr[ARGMINIUSB_FIFO_LEN] = {CommandID, AGMU_sCard_setup,0x00};

            SetCardDataStr[2] = GetN(&readerData[readerNum].card.atr);
            AgmLog2(AGM_LOG_DEBUG, "SetCardDataStr[2] = %x \n", SetCardDataStr[2]);
            retVal = WriteUSB(&readerData[readerNum], SetCardDataStr, AGM_DEFAULT_TIMEOUT);
            if (retVal <= 0)
            {
                AgmLog1(AGM_LOG_DEBUG, "=====Write T0 data failed!=============\n");
            }

			AgmLog1(AGM_LOG_DEBUG, "Parse read ATR success \n");
		}
		else if( AGM_ERROR_TIMEOUT == retVal )
		{
			AgmLog1(AGM_LOG_DEBUG, "Parse read ATR fail \n");
			pcscResponce = IFD_COMMUNICATION_ERROR;
		}
		else
		{
			AgmLog1(AGM_LOG_DEBUG,
					  "Argos Mini II Reader:IFDHPowerICC read ATR fail \n");
			pcscResponce = IFD_ERROR_POWER_ACTION;
        }
		break;
	case IFD_POWER_DOWN:
		AgmLog1(AGM_LOG_DEBUG, "Argos Mini II Reader:IFD_POWER_DOWN \n");

		/* Set default baudrate. */
		readerData[readerNum].CurrentDi = 1;

		retVal = CardPowerOff(&readerData[readerNum]);

		if (retVal <= 0)
		{
			AgmLog1(AGM_LOG_DEBUG,
					  "IFDHPowerICC - Error! while powering off card\n");
			pcscResponce = IFD_COMMUNICATION_ERROR;
		}

		readerData[readerNum].card.atr.length = 0;

		break;
	default:
		pcscResponce = IFD_COMMUNICATION_ERROR;
	}

	if (IFD_SUCCESS != pcscResponce)
	{
		memset(Atr, 0, MAX_ATR_SIZE);
		AtrLength = 0;
		readerData[readerNum].card.atr.length = 0;
		AgmLog1(AGM_LOG_DEBUG, "Argos Mini II Reader : IFDHPowerICC failed.\n");
	}
	unlock_mutex(&readerData[readerNum]);
#ifdef AGMII_DEBUG
	AgmLog1(AGM_LOG_DEBUG, "Argos Mini II Reader : exiting IFDHPowerICC\n");
#endif
	return pcscResponce;
}


/*****************************************************************************
*
*****************************************************************************/
RESPONSECODE IFDHTransmitToICC(DWORD Lun, SCARD_IO_HEADER SendPci,
										 PUCHAR TxBuffer, DWORD TxLength,
										 PUCHAR RxBuffer, PDWORD RxLength,
										 PSCARD_IO_HEADER RecvPci)
{

	/* This function performs an APDU exchange with the card/slot specified by
	 * Lun.  The driver is responsible for performing any protocol specific exchanges
	 * such as T=0/1 ... differences.  Calling this function will abstract all protocol
	 * differences.
	 *
	 * SendPci
	 * Protocol - 0, 1, .... 14
	 * Length   - Not used.
	 *
	 * TxBuffer - Transmit APDU example (0x00 0xA4 0x00 0x00 0x02 0x3F 0x00)
	 * TxLength - Length of this buffer.
	 * RxBuffer - Receive APDU example (0x61 0x14)
	 * RxLength - Length of the received APDU.  This function will be passed
	 * the size of the buffer of RxBuffer and this function is responsible for
	 * setting this to the length of the received APDU.  This should be ZERO
	 * on all errors.  The resource manager will take responsibility of zeroing
	 * out any temporary APDU buffers for security reasons.
	 *
	 * RecvPci
	 * Protocol - 0, 1, .... 14
	 * Length   - Not used.
	 *
	 * Notes:
	 * The driver is responsible for knowing what type of card it has.  If the current
	 * slot/card contains a memory card then this command should ignore the Protocol
	 * and use the MCT style commands for support for these style cards and transmit
	 * them appropriately.  If your reader does not support memory cards or you don't
	 * want to then ignore this.
	 *
	 * RxLength should be set to zero on error.
	 *
	 * returns:
	 *
	 * IFD_SUCCESS
	 * IFD_COMMUNICATION_ERROR
	 * IFD_RESPONSE_TIMEOUT
	 * IFD_ICC_NOT_PRESENT
	 * IFD_PROTOCOL_NOT_SUPPORTED
	 */
#ifdef AGMII_DEBUG
	AgmLog3(AGM_LOG_DEBUG,
			  "Argos Mini II Reader : entering IFDHTransmitToICC Lun = 0x%04x%04x\n",
			  (int) ((Lun >> 16) & 0xFFFF), (int) (Lun & 0xFFFF));
#endif

	int retVal, readerNum = (Lun & 0xFFFF0000) >> 16;

	if (TxBuffer == NULL || TxLength == 0)
	{
#ifdef AGMII_DEBUG
		AgmLog1(AGM_LOG_DEBUG,
				  "Argos Mini II Reader : IFDHTransmitToICC - Error - no data in command!!!\n");
#endif
		if (RxLength)
		{
			*RxLength = 0;
		}
		return IFD_COMMUNICATION_ERROR;
	}

	switch (SendPci.Protocol)
	{
	case ATR_PROTOCOL_TYPE_T0:
		AgmLog1(AGM_LOG_DEBUG,
				  "Argos Mini II :IFDHTransmitToICC ATR_PROTOCOL_TYPE_T0\n");
		lock_mutex(&readerData[readerNum]);
		retVal =
			AGMTransmitT0(&readerData[readerNum], TxBuffer, TxLength, RxBuffer,
							  RxLength);
		unlock_mutex(&readerData[readerNum]);
		AgmLog2(AGM_LOG_DEBUG, "AGMTransmitT0 return %d\n", retVal);
		break;

	case ATR_PROTOCOL_TYPE_T1:
		AgmLog1(AGM_LOG_DEBUG,
				  "Argos Mini II :IFDHTransmitToICC ATR_PROTOCOL_TYPE_T1\n");
		lock_mutex(&readerData[readerNum]);
		retVal =
			AGMTransmitT1(&readerData[readerNum], TxBuffer, TxLength, RxBuffer,
							  RxLength);
		unlock_mutex(&readerData[readerNum]);
		break;

	default:
		AgmLog1(AGM_LOG_DEBUG, "Argos Mini II :IFDHTransmitToICC default\n");
		if (RxLength)
			*RxLength = 0;
		return IFD_PROTOCOL_NOT_SUPPORTED;
	}
	switch (retVal)
	{
	case AGM_OK:
		return IFD_SUCCESS;
	case AGM_READER_NO_CARD_ERROR:
		return IFD_ICC_NOT_PRESENT;
    case AGM_ERROR_TIMEOUT :
        return IFD_RESPONSE_TIMEOUT;
	default:
		return IFD_COMMUNICATION_ERROR;
	}

	return IFD_SUCCESS;
}

/*****************************************************************************
*
*****************************************************************************/
RESPONSECODE IFDHControl(DWORD Lun,
								 PUCHAR TxBuffer, DWORD TxLength,
								 PUCHAR RxBuffer, PDWORD RxLength)
{


#ifdef AGMII_DEBUG
	AgmLog3(AGM_LOG_DEBUG,
			  "Argos Mini II Reader : entering IFDHControl Lun = 0x%04x%04x\n",
			  (int) ((Lun >> 16) & 0xFFFF), (int) (Lun & 0xFFFF));
#endif



	/* Select protocol */

	/* Write data to ICC */

	/* Read data from ICC */



	return IFD_SUCCESS;
}

/*****************************************************************************
*
*****************************************************************************/
RESPONSECODE IFDHICCPresence(DWORD Lun)
{

	/* This function returns the status of the card inserted in the
	 * reader/slot specified by Lun.  It will return either:
	 *
	 * returns:
	 * IFD_ICC_PRESENT
	 * IFD_ICC_NOT_PRESENT
	 * IFD_COMMUNICATION_ERROR
	 */

/*
#ifdef AGMII_DEBUG
	AgmLog3(AGM_LOG_DEBUG, "\n\
		==============================================\n\
		Argos Mini II Reader : entering IFDHICCPresence Lun = 0x%04x%04x\n\
		==============================================", (int) ((Lun >> 16) & 0xFFFF), (int) (Lun & 0xFFFF));
#endif
 */
	unsigned char data[8];
	int retVal, len, readerNum = (Lun & 0xFFFF0000) >> 16;

	retVal = GetStatus(&readerData[readerNum], (char *) data, &len);

	//AgmLog2(AGM_LOG_DEBUG, "GetStatus() returned \"%d\"", retVal);


	switch (retVal)
	{
	case 2:
		/*
		AgmLog2(AGM_LOG_DEBUG,
				  "IFDHICCPresence returning readerData[readerNum].card.status = %d",
				  readerData[readerNum].card.status);
		 */
		return readerData[readerNum].card.status;
	case 1:
		//AgmLog1(AGM_LOG_DEBUG, "IFDHICCPresence returning IFD_ICC_PRESENT");
		return IFD_ICC_PRESENT;
	case 0:
		//AgmLog1(AGM_LOG_DEBUG, "IFDHICCPresence returning IFD_ICC_NOT_PRESENT");
		return IFD_ICC_NOT_PRESENT;
	case -1:
		if (errno == ENODEV) {
			AgmLog1(AGM_LOG_DEBUG,
					  "IFDHICCPresence returning IFD_NO_SUCH_DEVICE");
			return IFD_NO_SUCH_DEVICE;
		}
	default:
		AgmLog1(AGM_LOG_DEBUG,
				  "IFDHICCPresence returning IFD_COMMUNICATION_ERROR");
		return IFD_COMMUNICATION_ERROR;
	}
}

