/*****************************************************************
 * File:		$RCSfile: usb.c,v $
 * Date:		$Date: 2005/05/27 09:59:19 $
 * Author:		$Author: kath $
 * Revision:	$Revision: 1.4 $
 *
 * Purpose:		This provides reader specific low-level calls
 * 				for the ArgosMini II (USB) reader of Todos Data System AB.
 * License:		See file LICENSE
 *******************************************************************/

#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <pcsclite.h>
#include "AgmII.h"


/*****************************************************************************
*
*****************************************************************************/
typedef struct
{
	int kMyVendorID;
	int kMyProductID;
} _usbID;

static _usbID UsbIDs[] = {
	{0x0B0C, 0x0009}
};


/*****************************************************************************
*
*****************************************************************************/
bool OpenUSB(reader * allReaders, reader * globalData)
{
	static struct usb_bus *busses = NULL;
	int id, id_number;
	struct usb_bus *bus;
	struct usb_dev_handle *dev_handle;

#ifdef AGMII_DEBUG
	int k;
	AgmLog1(AGM_LOG_DEBUG, "OpenUSB: ");
#endif

	if (busses == NULL)
	{
		usb_init();
	}

	usb_find_busses();
	usb_find_devices();

	busses = usb_get_busses();

	if (busses == NULL)
	{
#ifdef AGMII_DEBUG
		AgmLog1(AGM_LOG_ERROR, "No USB busses found");
#endif
		return FALSE;
	}

	if (globalData->io.handle != NULL)
	{
#ifdef AGMII_DEBUG
		AgmLog1(AGM_LOG_ERROR, "USB driver in use");
#endif
		return FALSE;
	}

	/* find any device corresponding to a UsbIDs entry */
	id_number = sizeof(UsbIDs) / sizeof(UsbIDs[0]);

	/* for any supported reader */
	for (id = 0; id < id_number; id++)
	{
		/* on any USB buses */
		for (bus = busses; bus; bus = bus->next)
		{
			struct usb_device *dev;

			/* any device on this bus */
			for (dev = bus->devices; dev; dev = dev->next)
			{
				if (dev->descriptor.idVendor == UsbIDs[id].kMyVendorID
					 && dev->descriptor.idProduct == UsbIDs[id].kMyProductID)
				{
					int r, already_used;
					char bus_device[BUS_DEVICE_STRSIZE];

					if (snprintf(bus_device, BUS_DEVICE_STRSIZE, "%s/%s",
									 bus->dirname, dev->filename) < 0)
					{
#ifdef AGMII_DEBUG
						AgmLog2(AGM_LOG_ERROR, "Device name too long: %s",
								  bus_device);
#endif
						return FALSE;
					}


					already_used = FALSE;
					for (r = 0; r < MAX_READER_NUM; r++)
					{
#ifdef AGMII_DEBUG
						AgmLog3(AGM_LOG_DEBUG,
								  "Checking new device '%s' against old '%s'",
								  bus_device, allReaders[r].io.bus_device);
#endif
						if (strcmp(allReaders[r].io.bus_device, bus_device) == 0)
						{
							already_used = TRUE;
						}
					}
					if (!already_used)
					{
						dev_handle = usb_open(dev);
						if (dev_handle)
						{
							int interface;
#ifdef AGMII_DEBUG
							AgmLog2(AGM_LOG_DEBUG,
									  "Trying to open USB bus/device: %s", bus_device);
#endif
							if (dev->config == NULL)
							{
#ifdef AGMII_DEBUG
								AgmLog2(AGM_LOG_ERROR, "No dev->config found for %s",
										  bus_device);
#endif
								return FALSE;
							}

							interface =
								dev->config[0].interface[0].altsetting[0].
								bInterfaceNumber;
							if (usb_claim_interface(dev_handle, interface) < 0)
							{
#ifdef AGMII_DEBUG
								AgmLog3(AGM_LOG_ERROR, "Can't claim interface %s: %s",
										  globalData->io.bus_device, strerror(errno));
#endif
								return FALSE;
							}

#ifdef AGMII_DEBUG
							AgmLog2(AGM_LOG_DEBUG, "Using USB bus/device: %s",
									  bus_device);

							for (k = 0;
								  k <
								  dev->config[0].interface[0].altsetting[0].
								  bNumEndpoints; k++)
							{
								AgmLog7(AGM_LOG_DEBUG,
										  "ep[%d] -> bDescriptorType = %X bEndpointAddress = %X bmAttributes = %X wMaxPacketSize = %X bLength = %x\n",
										  k,
										  dev->config[0].interface[0].altsetting[0].endpoint[k].bDescriptorType,
										  dev->config[0].interface[0].altsetting[0].endpoint[k].bEndpointAddress,
										  dev->config[0].interface[0].altsetting[0].endpoint[k].bmAttributes,
										  dev->config[0].interface[0].altsetting[0].endpoint[k].wMaxPacketSize,
										  dev->config[0].interface[0].altsetting[0].endpoint[k].bLength);
							}
#endif

							/* store device information */
							globalData->io.curPos = globalData->io.lastPos =
								globalData->io.stopReading = 0;
							globalData->io.handle = dev_handle;
							globalData->io.interface = dev->config[0].interface[0].altsetting[0].bInterfaceNumber;
							globalData->io.input_endpoint = dev->config[0].interface[0].altsetting[0].endpoint[USB_INEP].bEndpointAddress;
							globalData->io.output_endpoint = dev->config[0].interface[0].altsetting[0].endpoint[USB_OUTEP].bEndpointAddress,
							strncpy(globalData->io.bus_device, bus_device,
									  BUS_DEVICE_STRSIZE);
							return TRUE;
						}
						else
						{
#ifdef AGMII_DEBUG
							AgmLog3(AGM_LOG_CRITICAL, "Can't usb_open(%s): %s",
									  bus_device, strerror(errno));
#endif
						}
					}
					else
					{
#ifdef AGMII_DEBUG
						AgmLog2(AGM_LOG_DEBUG,
								  "USB device %s already in use. Checking next one.",
								  bus_device);
#endif
					}
				}
			}
		}
	}

	return FALSE;

}


/*****************************************************************************
*
*****************************************************************************/
int WriteUSB(reader * globalData, unsigned char *data, int timeOut)
{
#ifdef AGMII_WRITE_USB_DEBUG
	AgmLog1(AGM_LOG_DEBUG, "WriteUSB: Enter!");
#endif

	int writesize = usb_bulk_write(globalData->io.handle,
											 globalData->io.output_endpoint,
											 (char *) data, ARGMINIUSB_FIFO_LEN,
											 timeOut);

	if (writesize < 0)
	{
#ifdef AGMII_WRITE_USB_DEBUG
		AgmLog2(AGM_LOG_DEBUG, "WriteUSB: Error! (writesize = %d)\n",
				  writesize);
#endif
		return writesize;
	}

	return writesize;
}


/*****************************************************************************
*
*****************************************************************************/
int ReadUSB(reader * globalData, unsigned char *buf, int timeOut)
{
#ifdef AGMII_READ_USB_DEBUG
	AgmLog1(AGM_LOG_DEBUG, "ReadUSB: Enter!");
#endif

	int readsize;

	readsize = usb_bulk_read(globalData->io.handle,
									 globalData->io.input_endpoint,
									 (char *) buf, ARGMINIUSB_FIFO_LEN, timeOut);

	if (readsize < 0)
	{
#ifdef AGMII_READ_USB_DEBUG
		AgmLog2(AGM_LOG_DEBUG, "ReadUSB: Error!(readsize = %d)\n", readsize);
#endif

		readsize = usb_bulk_read(globalData->io.handle,
										 globalData->io.input_endpoint,
										 (char *) buf, ARGMINIUSB_FIFO_LEN, timeOut);

		if (readsize < 0)
		{
#ifdef AGMII_READ_USB_DEBUG
			AgmLog2(AGM_LOG_DEBUG, "ReadUSB: Error!(readsize = %d)\n", readsize);
#endif
			return readsize;
		}
	}

	return readsize;
}

//*****************************************************************************
//
//*****************************************************************************
void CleanReadBufferUSB(reader * globalData)
{
//    unsigned char temp[700];
//
//    usb_bulk_read(globalData->io.handle,
//                  globalData->io.input_endoint,
//                  temp, 300, 1000 /* 1 sec */);

	globalData->io.curPos = globalData->io.lastPos = 0;
}


/*****************************************************************************
*
*****************************************************************************/
bool CloseUSB(reader * globalData)
{
#ifdef AGMII_DEBUG
	AgmLog2(AGM_LOG_DEBUG, "CloseUSB: device = %s", globalData->io.bus_device);
#endif

	usb_release_interface(globalData->io.handle,
								 globalData->io.interface);
	usb_reset(globalData->io.handle);
	usb_close(globalData->io.handle);

	/* mark the resource unused */
	globalData->io.handle = NULL;
	globalData->io.bus_device[0] = '\0';
	globalData->io.stopReading = 1;

	return TRUE;
}

/*
   vim:set shiftwidth=3:
   vim:tabstop=3:
*/
